/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file implements classes SKGAccountObject.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgaccountobject.h"
#include "skgbankobject.h"
#include "skgoperationobject.h"
#include "skgunitobject.h"
#include "skginterestobject.h"
#include "skgdocumentbank.h"
#include "skgtraces.h"

#include <klocale.h>

SKGAccountObject::SKGAccountObject(const SKGDocument* iDocument, int iID): SKGNamedObject(iDocument, "v_account", iID) {}

SKGAccountObject::~SKGAccountObject() {}

SKGAccountObject::SKGAccountObject(const SKGAccountObject& iObject)
                : SKGNamedObject(iObject)
{}

SKGAccountObject::SKGAccountObject(const SKGNamedObject& iObject)
                : SKGNamedObject(iObject.getDocument(), "v_account", iObject.getID())
{
        if (iObject.getRealTable()=="account") {
                copyFrom(iObject);
        } else {
                *this=SKGNamedObject::SKGNamedObject(iObject.getDocument(), "v_account", iObject.getID());
        }
}

SKGAccountObject::SKGAccountObject(const SKGObjectBase& iObject)
{
        if (iObject.getRealTable()=="account") {
                copyFrom(iObject);
        } else {
                *this=SKGNamedObject::SKGNamedObject(iObject.getDocument(), "v_account", iObject.getID());
        }
}

const SKGAccountObject& SKGAccountObject::operator= (const SKGObjectBase& iObject)
{
        copyFrom(iObject);
        return *this;
}

SKGError SKGAccountObject::setBank(const SKGBankObject& iBank)
{
        return setAttribute("rd_bank_id", SKGServices::intToString(iBank.getID()));
}

SKGError SKGAccountObject::getBank(SKGBankObject& oBank) const
{
        SKGError err = getObject(getDocument(), "v_bank", "id=" + getAttribute("rd_bank_id"), oBank);
        return err;
}

SKGError SKGAccountObject::setNumber(const QString& iNumber)
{
        return setAttribute("t_number", iNumber);
}

QString SKGAccountObject::getNumber() const
{
        return getAttribute("t_number");
}

SKGError SKGAccountObject::setComment(const QString& iComment)
{
        return setAttribute("t_comment", iComment);
}

QString SKGAccountObject::getComment() const
{
        return getAttribute("t_comment");
}

SKGError SKGAccountObject::setAgencyNumber(const QString& iNumber)
{
        return setAttribute("t_agency_number", iNumber);
}

QString SKGAccountObject::getAgencyNumber() const
{
        return getAttribute("t_agency_number");
}

SKGError SKGAccountObject::setAgencyAddress(const QString& iAddress)
{
        return setAttribute("t_agency_address", iAddress);
}

QString SKGAccountObject::getAgencyAddress() const
{
        return getAttribute("t_agency_address");
}

SKGError SKGAccountObject::addOperation(SKGOperationObject& oOperation)
{
        SKGError err;
        if (getID() == 0) err=SKGError(ERR_FAIL, i18n("%1 failed because linked object is not yet saved in the database.",QString("SKGAccountObject::addOperation")));
        else {
                oOperation = SKGOperationObject(getDocument());
                err = oOperation.setParentAccount(*this);
        }
        return err;
}

double SKGAccountObject::getCurrentAmount() const
{
        return SKGServices::stringToDouble(getAttribute("f_CURRENTAMOUNT"));
}

double SKGAccountObject::getAmount(const QDate& iDate) const
{
        double output=0;
        SKGStringListList listTmp;
        SKGError err=SKGServices::executeSelectSqliteOrder(getDocument(),"SELECT TOTAL(f_QUANTITY), rc_unit_id FROM v_operation  WHERE "
                        "d_date<='"+SKGServices::dateToSqlString(QDateTime(iDate))+ "' AND rd_account_id="+SKGServices::intToString(getID())+
                        " GROUP BY rc_unit_id",
                        listTmp);
        int nb=listTmp.count();
        for (int i=1; err.isSucceeded() && i<nb ;++i) {
                QString quantity=listTmp.at(i).at(0);
                QString unitid=listTmp.at(i).at(1);

                double coef=1;
                QString val=getDocument()->getCachedValue("unitvalue-"+unitid);
                if (!val.isEmpty()) {
                        //Yes
                        coef=SKGServices::stringToDouble(val);
                } else {
                        //No
                        SKGUnitObject unit(getDocument(), SKGServices::stringToInt(unitid));
                        coef=unit.getAmount(iDate);
                }

                output+=coef*SKGServices::stringToDouble(quantity);
        }
        return output;
}

SKGError SKGAccountObject::setType(SKGAccountObject::AccountType iType)
{
        SKGError err;
        if (err.isSucceeded()) err=setAttribute("t_type", (iType==CURRENT ? "C" : (iType==CREDITCARD ? "D" : (iType==ASSETS ? "A" : (iType==INVESTMENT ? "I" : "O")))));
        return err;
}

SKGAccountObject::AccountType SKGAccountObject::getType() const
{
        QString typeString=getAttribute("t_type");
        return (typeString=="C" ? CURRENT : (typeString=="D" ? CREDITCARD : (typeString=="A" ? ASSETS : (typeString=="I" ? INVESTMENT : OTHER))));
}

SKGError SKGAccountObject::setClosed(bool iClosed)
{
        return setAttribute("t_close", iClosed ? "Y" :"N");
}

bool SKGAccountObject::isClosed() const
{
        return (getAttribute("t_close")=="Y" ? true : false);
}

SKGError SKGAccountObject::getUnit(SKGUnitObject& oUnit) const
{
        SKGObjectBase::SKGListSKGObjectBase units;
        SKGError err=SKGObjectBase::getObjects(getDocument(), "v_unit",
                                               "t_type IN ('1', '2', 'C') AND (SELECT count(1) from v_operation_display where rc_unit_id=v_unit.id AND rd_account_id="+SKGServices::intToString(getID())+")>0 ORDER BY t_type", units);
        int nb=units.count();
        if (nb) oUnit=units.at(0);
        return err;
}

SKGError SKGAccountObject::addInterest(SKGInterestObject& oInterest)
{
        SKGError err;
        if (getID() == 0) err=SKGError(ERR_FAIL, i18n("%1 failed because linked object is not yet saved in the database.",QString("SKGAccountObject::addInterest")));
        else {
                oInterest = SKGInterestObject((SKGDocumentBank*) getDocument());
                err = oInterest.setAttribute("rd_account_id", SKGServices::intToString(getID()));
        }
        return err;
}

SKGError SKGAccountObject::getInterests(SKGListSKGObjectBase& oInterestList) const
{
        SKGError err = getObjects(getDocument(), "v_interest",
                                  "rd_account_id=" + SKGServices::intToString(getID()),
                                  oInterestList);
        return err;
}

SKGError SKGAccountObject::getInterest(const QDate& iDate, SKGInterestObject& oInterest) const
{
        QString ids=SKGServices::intToString(getID());
        QString dates=SKGServices::dateToSqlString(QDateTime(iDate));
        SKGError err=SKGObjectBase::getObject(getDocument(), "v_interest",
                                              "rd_account_id=" + ids +" AND d_date<='"+dates+
                                              "' AND  ABS(strftime('%s','"+dates+
                                              "')-strftime('%s',d_date))=(select MIN(ABS(strftime('%s','"+dates+
                                              "')-strftime('%s',u2.d_date))) from interest u2 where u2.rd_account_id=" + ids +
                                              " AND u2.d_date<='"+dates+"')",
                                              oInterest);

        //If not found then get first
        if (err.isFailed()) err=SKGObjectBase::getObject(getDocument(), "v_interest",
                                        "rd_account_id=" + SKGServices::intToString(getID()) +" AND d_date=(select MIN(d_date) from interest where rd_account_id=" +
                                        SKGServices::intToString(getID()) + ')',
                                        oInterest);
        return err;
}

SKGError SKGAccountObject::getInterestItems(QList<SKGAccountObject::SKGInterestItem>& oInterestList, double& oInterests) const
{
        oInterestList.clear();
        SKGError err;
        QDate initialDate=QDate::currentDate();
        initialDate=initialDate.addMonths ( 1-initialDate.month() );
        initialDate=initialDate.addDays ( 1-initialDate.day() );

        oInterests=0;
        bool computationNeeded=false;

        //Add operations
        SKGObjectBase::SKGListSKGObjectBase items;
        err=SKGObjectBase::getObjects(getDocument(), "v_operation", "rd_account_id=" + SKGServices::intToString(getID()) +" AND d_date>='"+SKGServices::dateToSqlString(QDateTime(initialDate))+ "' ORDER BY d_date", items);
        int nb=items.count();
        for (int i=0; err.isSucceeded() && i<nb; ++i) {
                SKGOperationObject ob=items.at(i);

                SKGInterestItem itemI;
                itemI.object=ob;
                itemI.date=ob.getDate();
                itemI.valueDate=itemI.date;
                itemI.rate=0;
                itemI.base=0;
                itemI.coef=0;
                itemI.annualInterest=0;
                itemI.accruedInterest=0;
                itemI.amount=ob.getCurrentAmount();

                oInterestList.push_back(itemI);
        }

        //Add interest
        if (err.isSucceeded()) {
                err=SKGObjectBase::getObjects(getDocument(), "v_interest", "rd_account_id=" + SKGServices::intToString(getID()) +" AND d_date>='"+SKGServices::dateToSqlString(QDateTime(initialDate))+ "' ORDER BY d_date", items);

                int pos=0;
                int nb=items.count();
                for (int i=0; err.isSucceeded() && i<nb; ++i) {
                        SKGInterestObject ob=items.at(i);

                        SKGInterestItem itemI;
                        itemI.object=ob;
                        itemI.date=ob.getDate();
                        itemI.valueDate=itemI.date;
                        itemI.rate=ob.getRate();
                        itemI.base=SKGServices::stringToInt(ob.getAttribute("t_base"));
                        itemI.coef=0;
                        itemI.annualInterest=0;
                        itemI.accruedInterest=0;
                        itemI.amount=0;

                        int nb2=oInterestList.count();
                        for (int j=pos; err.isSucceeded() && j<nb2; ++j) {
                                if (itemI.date<=oInterestList.at(j).date) {
                                        break;
                                } else {
                                        ++pos;
                                }
                        }

                        oInterestList.insert(pos, itemI);
                        computationNeeded=true;
                }
        }

        //Get first interest
        if (err.isSucceeded()) {
                SKGInterestObject firstInterest;
                if (getInterest(initialDate, firstInterest).isSucceeded()) {
                        if (firstInterest.getDate()<initialDate) {
                                SKGInterestItem itemI;
                                itemI.object=firstInterest;
                                itemI.date=initialDate;
                                itemI.valueDate=initialDate;
                                itemI.rate=firstInterest.getRate();
                                itemI.base=0;
                                itemI.coef=0;
                                itemI.annualInterest=0;
                                itemI.accruedInterest=0;
                                itemI.amount=0;

                                oInterestList.insert(0, itemI);
                                computationNeeded=true;
                        }
                }
        }

        //Launch computation
        if (err.isSucceeded()) {
                if (computationNeeded) {

                        err=computeInterestItems(oInterestList, oInterests);
                } else {
                        //Drop temporary table
                        if (err.isSucceeded()) {
                                QString sql= "DROP TABLE IF EXISTS interest_result;;";
                                err = SKGServices::executeSqliteOrder(getDocument(), sql);
                        }
                }
        }
        return err;
}

SKGError SKGAccountObject::computeInterestItems(QList<SKGAccountObject::SKGInterestItem>& ioInterestList, double& oInterests) const
{
        SKGError err;

        //Sum annual interest
        oInterests=0;

        //Default interest item
        SKGInterestItem currentInterest;
        currentInterest.date=QDate::currentDate();
        currentInterest.date=currentInterest.date.addMonths ( 1-currentInterest.date.month() );
        currentInterest.date=currentInterest.date.addDays ( 1-currentInterest.date.day() );
        currentInterest.valueDate=currentInterest.date;
        currentInterest.rate=0;
        currentInterest.coef=0;
        currentInterest.annualInterest=0;
        currentInterest.accruedInterest=0;

        int nb=ioInterestList.count();
        for (int i=0; err.isSucceeded() && i<nb; ++i) {
                SKGInterestItem tmp=ioInterestList.at(i);
                SKGObjectBase object=tmp.object;
                if (object.getRealTable()=="operation") {
                        //Get operations
                        SKGOperationObject op=object;

                        //Get current amount
                        tmp.amount=op.getCurrentAmount();

                        //Get value date computation mode
                        SKGInterestObject::ValueDateMode valueMode=SKGInterestObject::FIFTEEN;
                        SKGInterestObject::InterestMode baseMode=SKGInterestObject::FIFTEEN24;
                        if (currentInterest.object.getRealTable()=="interest") {
                                SKGInterestObject interestObj=currentInterest.object;
                                valueMode=(tmp.amount>=0 ? interestObj.getIncomeValueDateMode() : interestObj.getExpenditueValueDateMode());
                                baseMode=interestObj.getInterestComputationMode();

                                tmp.rate=interestObj.getRate();
                        }

                        //Compute value date
                        if (object.getRealTable()=="operation") {
                                if (valueMode==SKGInterestObject::FIFTEEN) {
                                        if (tmp.amount>=0) {
                                                if (tmp.date.day()<=15) tmp.valueDate=tmp.date.addDays ( 16-tmp.date.day() );
                                                else tmp.valueDate=tmp.date.addMonths(1).addDays ( 1-tmp.date.day() );
                                        } else {
                                                if (tmp.date.day()<=15) tmp.valueDate=tmp.date.addDays ( 1-tmp.date.day() );
                                                else tmp.valueDate=tmp.date.addDays ( 16-tmp.date.day() );
                                        }
                                } else {
                                        tmp.valueDate=tmp.date.addDays ( tmp.amount>=0 ? ((int) valueMode)-1 : -((int) valueMode)+1 );
                                }
                        }

                        //Compute coef
                        if (baseMode==SKGInterestObject::DAYS365) {
                                QDate last(tmp.date.year(), 12, 31);
                                tmp.coef=tmp.valueDate.daysTo(last)+1;
                                tmp.coef/=365;
                        } else if (baseMode==SKGInterestObject::DAYS360) {
                                QDate last(tmp.date.year(), 12, 31);
                                tmp.coef=360*(last.year()-tmp.valueDate.year())+30*(last.month()-tmp.valueDate.month())+(last.day()-tmp.valueDate.day()); //TODO must be confirmed
                                tmp.coef/=360;
                        } else {
                                tmp.coef=2*(12-tmp.valueDate.month())+(tmp.valueDate.day()<=15 ? 2 : 1);
                                tmp.coef/=24;
                        }

                        //Compute annual interest
                        tmp.annualInterest=tmp.amount*tmp.coef*tmp.rate/100;

                } else if (object.getRealTable()=="interest") {
                        //Compute coef
                        if (tmp.base==365) {
                                QDate last(tmp.date.year(), 12, 31);
                                tmp.coef=tmp.valueDate.daysTo(last)+1;
                                tmp.coef/=365;
                        } else if (tmp.base==360) {
                                QDate last(tmp.date.year(), 12, 31);
                                tmp.coef=360*(last.year()-tmp.valueDate.year())+30*(last.month()-tmp.valueDate.month())+(last.day()-tmp.valueDate.day()); //TODO must be confirmed
                                tmp.coef/=360;
                        } else {
                                tmp.coef=2*(12-tmp.valueDate.month())+(tmp.valueDate.day()<=15 ? 2 : 1);
                                tmp.coef/=24;
                        }

                        //Compute annual interest
                        tmp.amount=getAmount(tmp.valueDate);
                        tmp.annualInterest=tmp.amount*tmp.coef*(tmp.rate-currentInterest.rate)/100;

                        currentInterest=tmp;
                }
                //Compute sum
                oInterests+=tmp.annualInterest;

                //Compute accrued interest
                tmp.accruedInterest=oInterests-getAmount(tmp.date)*tmp.coef*tmp.rate/100;

                ioInterestList[i]=tmp;
        }

        //Create temporary table
        if (err.isSucceeded()) {
                QString sql=
                        "DROP TABLE IF EXISTS interest_result;;"
                        "CREATE TEMP TABLE interest_result("
                        "id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,"
                        "d_date DATE NOT NULL,"
                        "d_valuedate DATE NOT NULL,"
                        "t_comment TEXT NOT NULL DEFAULT '',"
                        "f_currentamount FLOAT NOT NULL DEFAULT 0,"
                        "f_coef FLOAT NOT NULL DEFAULT 0,"
                        "f_rate FLOAT NOT NULL DEFAULT 0,"
                        "f_annual_interest FLOAT NOT NULL DEFAULT 0,"
                        "f_accrued_interest FLOAT NOT NULL DEFAULT 0"
                        ");;";
                err = SKGServices::executeSqliteOrder(getDocument(), sql);

                //Fill table
                int nb=ioInterestList.count();
                for (int i=0; err.isSucceeded() && i<nb; ++i) {
                        SKGInterestItem interest=ioInterestList.at(i);
                        SKGObjectBase object=interest.object;
                        QString sqlinsert=
                                "INSERT INTO interest_result (d_date,d_valuedate,t_comment,f_currentamount,f_coef,f_rate,f_annual_interest,f_accrued_interest) "
                                " VALUES ('"+SKGServices::dateToSqlString(QDateTime(interest.date))+
                                "','"+SKGServices::dateToSqlString(QDateTime(interest.valueDate))+
                                "','"+SKGServices::stringToSqlString(object.getRealTable()=="operation" ? i18n("Relative to operation [%1]", SKGOperationObject(object).getDisplayName()): i18n("Rate change"))+
                                "',"+SKGServices::doubleToString(interest.amount)+
                                ','+SKGServices::doubleToString(interest.coef)+
                                ','+SKGServices::doubleToString(interest.rate)+
                                ','+SKGServices::doubleToString(interest.annualInterest)+
                                ','+SKGServices::doubleToString(interest.accruedInterest)+
                                ");;";

                        err = SKGServices::executeSqliteOrder(getDocument(), sqlinsert);

                }
        }
        return err;
}

#include "skgaccountobject.moc"
