/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * A skrooge plugin to calculate.
 *
 * @author Stephane MANKOWSKI
 */
#include "skgcalculatorpluginwidget.h"
#include "skgmainpanel.h"
#include "skgtraces.h"
#include "skgdocumentbank.h"
#include "skgobjectmodel.h"
#include "skginterestobject.h"
#include "skgaccountobject.h"
#include "skgtransactionmng.h"

#include <kstandarddirs.h>

#include <QDomDocument>
#include <QDir>

SKGCalculatorPluginWidget::SKGCalculatorPluginWidget(SKGDocument* iDocument)
                :SKGTabPage(iDocument)
{
        SKGTRACEIN(1, "SKGCalculatorPluginWidget::SKGCalculatorPluginWidget");

        ui.setupUi(this);

        ui.kUpdate->setIcon ( KIcon ( "dialog-ok-apply" ) );
        ui.kAdd->setIcon ( KIcon ( "list-add" ) );

//         ui.kCalculatorBtn->setIcon(KIcon("accessories-calculator"));
//         ui.kInterestBtn->setIcon(KIcon("skrooge_more"));

        ui.kCreditValueDate->addItem ( i18n ( "Fifteen" ) );
        ui.kCreditValueDate->addItem ( i18n ( "Day +0" ) );
        ui.kCreditValueDate->addItem ( i18n ( "Day +1" ) );
        ui.kCreditValueDate->addItem ( i18n ( "Day +2" ) );
        ui.kCreditValueDate->addItem ( i18n ( "Day +3" ) );
        ui.kCreditValueDate->addItem ( i18n ( "Day +4" ) );
        ui.kCreditValueDate->addItem ( i18n ( "Day +5" ) );

        ui.kDebitValueDate->addItem ( i18n ( "Fifteen" ) );
        ui.kDebitValueDate->addItem ( i18n ( "Day -0" ) );
        ui.kDebitValueDate->addItem ( i18n ( "Day -1" ) );
        ui.kDebitValueDate->addItem ( i18n ( "Day -2" ) );
        ui.kDebitValueDate->addItem ( i18n ( "Day -3" ) );
        ui.kDebitValueDate->addItem ( i18n ( "Day -4" ) );
        ui.kDebitValueDate->addItem ( i18n ( "Day -5" ) );

        ui.kMode->addItem ( i18n ( "24 fifteen" ) );
        ui.kMode->addItem ( i18n ( "360 days" ) );
        ui.kMode->addItem ( i18n ( "365 days" ) );

        //Bind account creation view
        objectModel = new SKGObjectModel ( ( SKGDocumentBank* ) getDocument(), "v_interest", "", this, "", false);
        ui.kInterestView->setModel ( objectModel );

        SKGObjectModel* resultModel = new SKGObjectModel ( ( SKGDocumentBank* ) getDocument(), "interest_result", "", this, "", false);
        ui.kInterestResultTable->setModel(resultModel);

        connect ( ui.kInterestView, SIGNAL ( selectionChangedDelayed() ), this, SLOT ( onSelectionChanged() ) );
        connect ( objectModel, SIGNAL ( beforeReset() ), ui.kInterestView, SLOT ( saveSelection() ) );
        connect ( objectModel, SIGNAL ( afterReset() ), ui.kInterestView, SLOT ( resetSelection() ) );

        //Build you panel here
//         setCurrentMode(-1);

        //Refresh
        connect ( ui.kDisplayAccountCombo, SIGNAL ( currentIndexChanged ( int ) ), this, SLOT ( onFilterChanged() ), Qt::QueuedConnection );

        connect ( ( const QObject* ) getDocument(), SIGNAL(tableModified(QString, int)), this, SLOT(dataModified(QString, int)), Qt::QueuedConnection );
        dataModified("", 0);
}

SKGCalculatorPluginWidget::~SKGCalculatorPluginWidget()
{
        SKGTRACEIN(1, "SKGCalculatorPluginWidget::~SKGCalculatorPluginWidget");
        objectModel=NULL;
}

// /*int SKGCalculatorPluginWidget::getCurrentMode()
// {
//         return mode;
// }*/

// void SKGCalculatorPluginWidget::onBtnModeClicked()
// {
//         QWidget* sender=static_cast<QWidget*> ( this->sender() );
//
//         int currentMode=getCurrentMode();
//         int newMode=0;
//
// //         if (sender==ui.kCalculatorBtn) newMode=0;
// //         else if (sender==ui.kInterestBtn) newMode=1;
//
//         if (currentMode==newMode) newMode=-1;
//         setCurrentMode(newMode);
// }
//
// void SKGCalculatorPluginWidget::setCurrentMode(int iMode)
// {
//         mode=iMode;
//         if (mode==-1) mode=0;
//
//         ui.kCalculatorFrm->hide();
//         ui.kInterestFrm->hide();
//
// //         ui.kCalculatorBtn->setChecked(false);
// //         ui.kInterestBtn->setChecked(false);
//
//         if (mode==0) {
//                 ui.kCalculatorFrm->show();
// //                 ui.kCalculatorBtn->setChecked(true);
//         } else if (mode==1) {
//                 ui.kInterestFrm->show();
// //                 ui.kInterestBtn->setChecked(true);
//         }
// }

QString SKGCalculatorPluginWidget::getState()
{
        SKGTRACEIN(10, "SKGCalculatorPluginWidget::getState");
        QDomDocument doc("SKGML");
        QDomElement root = doc.createElement("parameters");
        doc.appendChild(root);

//         root.setAttribute ( "currentPage", SKGServices::intToString(getCurrentMode()) );
        root.setAttribute ( "account", ui.kDisplayAccountCombo->currentText() );

        //Memorize table settings
        root.setAttribute ( "view", ui.kInterestView->getState() );
        root.setAttribute ( "viewResult", ui.kInterestResultTable->getState() );

        return doc.toString();
}

void SKGCalculatorPluginWidget::setState(const QString& iState )
{
        SKGTRACEIN(10, "SKGCalculatorPluginWidget::setState");
        QDomDocument doc("SKGML");
        doc.setContent(iState);
        QDomElement root = doc.documentElement();

        QString account=root.attribute ( "account" );
        QString currentPage=root.attribute ( "currentPage" );
//         if ( !currentPage.isEmpty() ) setCurrentMode ( SKGServices::stringToInt ( currentPage ) );
        if ( !account.isEmpty() ) {
                bool previous=ui.kDisplayAccountCombo->blockSignals(true);
                ui.kDisplayAccountCombo->setText ( account );
                ui.kDisplayAccountCombo->blockSignals(previous);
        }

        //Update model
        if (objectModel) {
                bool previous=objectModel->blockRefresh(true);
                onFilterChanged();
                objectModel->blockRefresh(previous);
        }

        //!!! Must be done here after onFilterChanged
        ui.kInterestView->setState ( root.attribute ( "view" ) );
        ui.kInterestResultTable->setState ( root.attribute ( "viewResult" ) );
}

QString SKGCalculatorPluginWidget::getDefaultStateAttribute()
{
        return "SKGCALCULATOR_DEFAULT_PARAMETERS";
}

QWidget* SKGCalculatorPluginWidget::getWidgetForPrint()
{
        return this;
}

void SKGCalculatorPluginWidget::onSelectionChanged()
{
        SKGTRACEIN ( 10, "SKGCalculatorPluginWidget::onSelectionChanged" );
        //Mapping
        QItemSelectionModel *selModel=ui.kInterestView->selectionModel();
        if ( selModel ) {
                QModelIndexList indexes=selModel->selectedRows();
                if ( indexes.count() && objectModel ) {
                        QModelIndex idx=indexes[indexes.count()-1];

                        SKGInterestObject interest ( objectModel->getObject ( idx ) );

                        ui.kDateEdit->setDate(interest.getDate());
                        ui.kRateEdit->setValue(interest.getRate());
                        ui.kCreditValueDate->setCurrentIndex((int) interest.getIncomeValueDateMode());
                        ui.kDebitValueDate->setCurrentIndex((int) interest.getExpenditueValueDateMode());
                        ui.kMode->setCurrentIndex((int) interest.getInterestComputationMode());
                }

                emit selectionChanged();
        }
}

void SKGCalculatorPluginWidget::dataModified(const QString& iTableName, int iIdTransaction)

{
        SKGTRACEIN ( 10, "SKGCalculatorPluginWidget::dataModified" );
        Q_UNUSED(iIdTransaction);
        QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );

        //Refresh widgets
        QSqlDatabase* db = getDocument()->getDatabase();
        setEnabled ( db!=NULL );
        if ( db!=NULL && (iTableName=="interest" || iTableName.isEmpty())) {
                //Correction bug 2299394 vvv
                if ( ui.kInterestView->isAutoResized() ) ui.kInterestView->resizeColumnsToContents();
                //Correction bug 2299394 ^^^

		//Correction 215658: vvv because the table is modified, the selection is modified
		onSelectionChanged();
		//Correction 215658: ^^^
        }

        //Refresh interest parameter view
        QString current=ui.kDisplayAccountCombo->currentText();
        if ( db!=NULL && (iTableName=="account" || iTableName.isEmpty())) {
                //Clear
                ui.kDisplayAccountCombo->clear();

                SKGStringListList listAccount;
                SKGServices::executeSelectSqliteOrder(getDocument(), QString("SELECT t_ICON, t_name from v_account_display where t_close='N' order by t_name"), listAccount);

                int nbAccounts=listAccount.count();
                if ( nbAccounts>=0 ) {
                        QDir dirLogo ( KStandardDirs::locate ( "data", QString::fromLatin1 ( "skrooge/images/logo/" ) ) );
                        for (int i=1; i<nbAccounts; ++i) { //Ignore header
                                QIcon icon( dirLogo.absoluteFilePath ( listAccount.at(i).at(0) ) );
                                QString text=listAccount.at(i).at(1);
                                ui.kDisplayAccountCombo->addItem (icon, text);
                        }
                }

                int pos=ui.kDisplayAccountCombo->findText ( current );
                if ( pos==-1 ) pos=0;
                ui.kDisplayAccountCombo->setCurrentIndex ( pos );

                ui.kAdd->setEnabled(ui.kDisplayAccountCombo->count()>0);
        }

        //Refresh computation
        if ( db!=NULL && (iTableName=="interest" ||iTableName=="account" || iTableName.isEmpty())) {
                computeInterest();
        }
        QApplication::restoreOverrideCursor();
}

void SKGCalculatorPluginWidget::computeInterest()
{
        //Compute
        SKGAccountObject account(getDocument());
        SKGError err=account.setName(ui.kDisplayAccountCombo->currentText());
        if (err.isSucceeded()) err=account.load();
        QList<SKGAccountObject::SKGInterestItem> oInterestList;
        double oInterests=0;
        if (err.isSucceeded()) err=account.getInterestItems(oInterestList, oInterests);
        if (err.isSucceeded()) {
                //Refresh table
                ui.kInterestResultTable->setState(ui.kInterestResultTable->getState());

                //Set text
                SKGDocumentBank* doc=( SKGDocumentBank* ) getDocument();
                SKGServices::SKGUnitInfo unit1=doc->getPrimaryUnit();
                SKGServices::SKGUnitInfo unit2=doc->getSecondaryUnit();

                QString s1=doc->formatMoney(oInterests, unit1);
                ui.kInterestLbl->setText ( i18n ( "Annual interest=%1", s1 ) );
                if ( !unit2.Name.isEmpty() && unit2.Value ) {
                        s1=doc->formatMoney(oInterests, unit2);
                        ui.kInterestLbl->setToolTip ( i18n ( "Annual interest=%1", s1 ) );
                }
        }
}

void SKGCalculatorPluginWidget::onFilterChanged()
{
        SKGTRACEIN ( 1, "SKGCalculatorPluginWidget::onFilterChanged" );
        if ( !isEnabled() ) return;
        QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );

        //Compute where clause
        QString account=ui.kDisplayAccountCombo->currentText();
        QString filter2="t_ACCOUNT='"+SKGServices::stringToSqlString ( account ) +'\'';

        //Update model
        if ( objectModel ) {
                objectModel->setFilter ( filter2 );
                ui.kInterestView->setState ( ui.kInterestView->getState() );

                //Compute result
                computeInterest();
        }
        QApplication::restoreOverrideCursor();
}

SKGObjectBase::SKGListSKGObjectBase SKGCalculatorPluginWidget::getSelectedObjects()
{
        return ui.kInterestView->getSelectedObjects();
}

int SKGCalculatorPluginWidget::getNbSelectedObjects()
{
        return ui.kInterestView->getNbSelectedObjects();
}

// void SKGCalculatorPluginWidget::onCalculatorReturnPressed(const QString& iText)
// {
// //         ui.kCalculatorList->addItem(iText);
// }

// /*void SKGCalculatorPluginWidget::onCalculatorListClicked(const QString& iText)
// {
// //         ui.kCalculatorEdit->setText(iText);
// //         ui.kCalculatorEdit->setFocus();
// }*/

void SKGCalculatorPluginWidget::onSelectedInterestChanged()
{
        int nbSel=getNbSelectedObjects();
        ui.kDateEdit->setEnabled(nbSel<=1);
        ui.kRateEdit->setEnabled(nbSel<=1);
        ui.kAdd->setEnabled(nbSel<=1);
}

void SKGCalculatorPluginWidget::onAdd()
{
        SKGError err;
        SKGTRACEINRC ( 10, "SKGCalculatorPluginWidget::onAdd",err );
        QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
        {
                SKGInterestObject interestObj;
                {
                        QString accountname=ui.kDisplayAccountCombo->currentText();
                        SKGBEGINTRANSACTION ( *getDocument(), i18n ( "Interest parameter creation for account [%1]", accountname ), err );

                        //Get Parent account
                        SKGAccountObject accountObj ( getDocument() );
                        if ( err.isSucceeded() ) err=accountObj.setName(accountname);
                        if ( err.isSucceeded() ) err=accountObj.load();

                        //Create interest object
                        if ( err.isSucceeded() ) err=accountObj.addInterest ( interestObj );
                        if ( err.isSucceeded() ) err=interestObj.setDate ( ui.kDateEdit->date() );
                        if ( err.isSucceeded() ) err=interestObj.setRate ( ui.kRateEdit->value() );
                        if ( err.isSucceeded() ) err=interestObj.setIncomeValueDateMode ( (SKGInterestObject::ValueDateMode) ui.kCreditValueDate->currentIndex() );
                        if ( err.isSucceeded() ) err=interestObj.setExpenditueValueDateMode ( (SKGInterestObject::ValueDateMode) ui.kDebitValueDate->currentIndex() );
                        if ( err.isSucceeded() ) err=interestObj.setInterestComputationMode ( (SKGInterestObject::InterestMode) ui.kMode->currentIndex() );
                        if ( err.isSucceeded() ) err=interestObj.save();
                }

                //status bar
                if ( err.isSucceeded() ) {
                        err=SKGError ( 0, i18n ( "Interest parameter created") );
                        ui.kInterestView->selectObject ( interestObj.getUniqueID() );
                } else err.addError ( ERR_FAIL, i18n ( "Interest parameter creation failed" ) );
        }
        QApplication::restoreOverrideCursor();

        //Display error
        SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

void SKGCalculatorPluginWidget::onUpdate()
{
        SKGError err;
        SKGTRACEINRC ( 10, "SKGCalculatorPluginWidget::onUpdate",err );
        QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );

        //Get Parent account
        QString accountname=ui.kDisplayAccountCombo->currentText();
        SKGAccountObject accountObj ( getDocument() );
        if ( err.isSucceeded() ) err=accountObj.setName(accountname);
        if ( err.isSucceeded() ) err=accountObj.load();

        SKGObjectBase::SKGListSKGObjectBase updatedInterest=getSelectedObjects();
        int nb=updatedInterest.count();
        {
                SKGInterestObject interestObj;
                {
                        SKGBEGINTRANSACTION ( *getDocument(), i18n ( "Interest parameter update for account [%1]", accountname ), err );

                        for (int i=0; err.isSucceeded() && i<nb; ++i) {
                                interestObj=updatedInterest.at(i);
                                if (nb==1) {
                                        if ( err.isSucceeded() ) err=interestObj.setDate ( ui.kDateEdit->date() );
                                        if ( err.isSucceeded() ) err=interestObj.setRate ( ui.kRateEdit->value() );
                                }
                                if ( err.isSucceeded() ) err=interestObj.setIncomeValueDateMode ( (SKGInterestObject::ValueDateMode) ui.kCreditValueDate->currentIndex() );
                                if ( err.isSucceeded() ) err=interestObj.setExpenditueValueDateMode ( (SKGInterestObject::ValueDateMode) ui.kDebitValueDate->currentIndex() );
                                if ( err.isSucceeded() ) err=interestObj.setInterestComputationMode ( (SKGInterestObject::InterestMode) ui.kMode->currentIndex() );
                                if ( err.isSucceeded() ) err=interestObj.save();
                        }
                }

                //status bar
                if ( err.isSucceeded() ) err=SKGError ( 0, i18n ( "Interest parameter updated") );
                else err.addError ( ERR_FAIL, i18n ( "Interest parameter update failed" ) );
        }
        QApplication::restoreOverrideCursor();

        //Display error
        SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}
#include "skgcalculatorpluginwidget.moc"


